import pytest

from wcs import data_sources
from wcs.admin.settings import UserFieldsFormDef
from wcs.data_sources import NamedDataSource
from wcs.fields import StringField
from wcs.qommon.http_request import HTTPRequest
from wcs.qommon.template import Template

from .utilities import clean_temporary_pub, create_temporary_pub


@pytest.fixture
def pub():
    pub = create_temporary_pub()
    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    return pub


def teardown_module(module):
    clean_temporary_pub()


def test_datasource_users(pub):
    pub.role_class.wipe()
    role1 = pub.role_class(name='role')
    role1.store()
    role2 = pub.role_class(name='role2')
    role2.store()

    pub.user_class.wipe()

    NamedDataSource.wipe()
    datasource = NamedDataSource(name='foo')
    datasource.data_source = {'type': 'wcs:users'}
    datasource.store()

    context = pub.substitutions.get_context_variables(mode='lazy')
    tmpl = Template(
        '{% for user in data_source.foo %}{{ user.text }}{% if not forloop.last %}, {% endif %}{% endfor %}'
    )

    assert data_sources.get_items({'type': datasource.slug}) == []
    assert data_sources.get_items(datasource.extended_data_source) == []
    assert data_sources.get_structured_items({'type': datasource.slug}) == []
    assert data_sources.get_structured_items(datasource.extended_data_source) == []
    assert tmpl.render(context) == ''

    users = []
    for i in range(2):
        user = pub.user_class(name='John Doe %s' % i)
        user.roles = [role1.id]
        user.name_identifiers = ['abc' + str(i)]
        user.store()
        users.append(user)

    assert data_sources.get_items({'type': datasource.slug}) == [
        (
            '1',
            'John Doe 0',
            '1',
            {
                'id': 1,
                'text': 'John Doe 0',
                'user_name_identifier_0': 'abc0',
                'user_nameid': 'abc0',
                'user_admin_access': False,
                'user_backoffice_access': False,
                'user_display_name': 'John Doe 0',
                'user_email': None,
            },
        ),
        (
            '2',
            'John Doe 1',
            '2',
            {
                'id': 2,
                'text': 'John Doe 1',
                'user_name_identifier_0': 'abc1',
                'user_nameid': 'abc1',
                'user_admin_access': False,
                'user_backoffice_access': False,
                'user_display_name': 'John Doe 1',
                'user_email': None,
            },
        ),
    ]
    assert data_sources.get_items(datasource.extended_data_source) == [
        (
            '1',
            'John Doe 0',
            '1',
            {
                'id': 1,
                'text': 'John Doe 0',
                'user_name_identifier_0': 'abc0',
                'user_nameid': 'abc0',
                'user_admin_access': False,
                'user_backoffice_access': False,
                'user_display_name': 'John Doe 0',
                'user_email': None,
            },
        ),
        (
            '2',
            'John Doe 1',
            '2',
            {
                'id': 2,
                'text': 'John Doe 1',
                'user_name_identifier_0': 'abc1',
                'user_nameid': 'abc1',
                'user_admin_access': False,
                'user_backoffice_access': False,
                'user_display_name': 'John Doe 1',
                'user_email': None,
            },
        ),
    ]
    assert data_sources.get_structured_items({'type': datasource.slug}) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        },
        {
            'id': 2,
            'text': 'John Doe 1',
            'user_name_identifier_0': 'abc1',
            'user_nameid': 'abc1',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 1',
            'user_email': None,
        },
    ]
    assert data_sources.get_structured_items(datasource.extended_data_source) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        },
        {
            'id': 2,
            'text': 'John Doe 1',
            'user_name_identifier_0': 'abc1',
            'user_nameid': 'abc1',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 1',
            'user_email': None,
        },
    ]
    assert tmpl.render(context) == 'John Doe 0, John Doe 1'

    datasource.users_included_roles = [role1.id]
    datasource.store()
    assert data_sources.get_structured_items({'type': datasource.slug}) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        },
        {
            'id': 2,
            'text': 'John Doe 1',
            'user_name_identifier_0': 'abc1',
            'user_nameid': 'abc1',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 1',
            'user_email': None,
        },
    ]
    assert data_sources.get_structured_items(datasource.extended_data_source) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        },
        {
            'id': 2,
            'text': 'John Doe 1',
            'user_name_identifier_0': 'abc1',
            'user_nameid': 'abc1',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 1',
            'user_email': None,
        },
    ]
    assert tmpl.render(context) == 'John Doe 0, John Doe 1'

    datasource.users_included_roles = [role1.id, role2.id]
    datasource.store()
    assert data_sources.get_structured_items({'type': datasource.slug}) == []
    assert data_sources.get_structured_items(datasource.extended_data_source) == []
    assert tmpl.render(context) == ''

    users[0].roles = [role1.id, role2.id]
    users[0].store()
    assert data_sources.get_structured_items({'type': datasource.slug}) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        }
    ]
    assert data_sources.get_structured_items(datasource.extended_data_source) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        }
    ]
    assert tmpl.render(context) == 'John Doe 0'

    users[0].roles = [role2.id]
    users[0].store()
    datasource.users_included_roles = []
    datasource.users_excluded_roles = [role1.id]
    datasource.store()
    assert data_sources.get_structured_items({'type': datasource.slug}) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        }
    ]
    assert data_sources.get_structured_items(datasource.extended_data_source) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        }
    ]
    assert tmpl.render(context) == 'John Doe 0'

    datasource.users_excluded_roles = [role1.id, role2.id]
    datasource.store()
    assert data_sources.get_structured_items({'type': datasource.slug}) == []
    assert data_sources.get_structured_items(datasource.extended_data_source) == []
    assert tmpl.render(context) == ''

    datasource.users_excluded_roles = []
    datasource.store()
    users[1].is_active = False
    users[1].store()
    assert not datasource.include_disabled_users
    assert data_sources.get_structured_items({'type': datasource.slug}) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        }
    ]
    assert data_sources.get_structured_items(datasource.extended_data_source) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        }
    ]
    assert tmpl.render(context) == 'John Doe 0'

    datasource.include_disabled_users = True
    datasource.store()
    assert data_sources.get_structured_items({'type': datasource.slug}) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        },
        {
            'id': 2,
            'text': 'John Doe 1',
            'user_name_identifier_0': 'abc1',
            'user_nameid': 'abc1',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 1',
            'user_email': None,
        },
    ]
    assert data_sources.get_structured_items(datasource.extended_data_source) == [
        {
            'id': 1,
            'text': 'John Doe 0',
            'user_name_identifier_0': 'abc0',
            'user_nameid': 'abc0',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 0',
            'user_email': None,
        },
        {
            'id': 2,
            'text': 'John Doe 1',
            'user_name_identifier_0': 'abc1',
            'user_nameid': 'abc1',
            'user_admin_access': False,
            'user_backoffice_access': False,
            'user_display_name': 'John Doe 1',
            'user_email': None,
        },
    ]
    assert tmpl.render(context) == 'John Doe 0, John Doe 1'

    # by uuid
    assert datasource.get_structured_value('abc0') == {
        'id': 1,
        'text': 'John Doe 0',
        'user_name_identifier_0': 'abc0',
        'user_nameid': 'abc0',
        'user_admin_access': False,
        'user_backoffice_access': False,
        'user_display_name': 'John Doe 0',
        'user_email': None,
    }
    assert datasource.get_display_value('abc0') == 'John Doe 0'
    assert datasource.get_structured_value('abc1') == {
        'id': 2,
        'text': 'John Doe 1',
        'user_name_identifier_0': 'abc1',
        'user_nameid': 'abc1',
        'user_admin_access': False,
        'user_backoffice_access': False,
        'user_display_name': 'John Doe 1',
        'user_email': None,
    }
    assert datasource.get_display_value('abc1') == 'John Doe 1'

    # by id
    assert datasource.get_structured_value('1') == {
        'id': 1,
        'text': 'John Doe 0',
        'user_name_identifier_0': 'abc0',
        'user_nameid': 'abc0',
        'user_admin_access': False,
        'user_backoffice_access': False,
        'user_display_name': 'John Doe 0',
        'user_email': None,
    }
    assert datasource.get_display_value('1') == 'John Doe 0'
    assert datasource.get_structured_value('2') == {
        'id': 2,
        'text': 'John Doe 1',
        'user_name_identifier_0': 'abc1',
        'user_nameid': 'abc1',
        'user_admin_access': False,
        'user_backoffice_access': False,
        'user_display_name': 'John Doe 1',
        'user_email': None,
    }
    assert datasource.get_display_value('2') == 'John Doe 1'

    # by numeric id
    assert datasource.get_structured_value(1) == {
        'id': 1,
        'text': 'John Doe 0',
        'user_name_identifier_0': 'abc0',
        'user_nameid': 'abc0',
        'user_admin_access': False,
        'user_backoffice_access': False,
        'user_display_name': 'John Doe 0',
        'user_email': None,
    }
    assert datasource.get_display_value(1) == 'John Doe 0'
    assert datasource.get_structured_value(2) == {
        'id': 2,
        'text': 'John Doe 1',
        'user_name_identifier_0': 'abc1',
        'user_nameid': 'abc1',
        'user_admin_access': False,
        'user_backoffice_access': False,
        'user_display_name': 'John Doe 1',
        'user_email': None,
    }
    assert datasource.get_display_value(2) == 'John Doe 1'

    datasource.users_included_roles = [role1.id]
    datasource.users_excluded_roles = [role2.id]
    datasource.store()
    users[0].roles = [role1.id, role2.id]
    users[0].store()

    # by uuid
    assert datasource.get_structured_value('abc0') is None
    assert datasource.get_display_value('abc0') is None
    assert datasource.get_structured_value('abc1') == {
        'id': 2,
        'text': 'John Doe 1',
        'user_name_identifier_0': 'abc1',
        'user_nameid': 'abc1',
        'user_admin_access': False,
        'user_backoffice_access': False,
        'user_display_name': 'John Doe 1',
        'user_email': None,
    }
    assert datasource.get_display_value('abc1') == 'John Doe 1'

    # by id
    assert datasource.get_structured_value('1') is None
    assert datasource.get_display_value('1') is None
    assert datasource.get_structured_value('2') == {
        'id': 2,
        'text': 'John Doe 1',
        'user_name_identifier_0': 'abc1',
        'user_nameid': 'abc1',
        'user_admin_access': False,
        'user_backoffice_access': False,
        'user_display_name': 'John Doe 1',
        'user_email': None,
    }
    assert datasource.get_display_value('2') == 'John Doe 1'

    datasource.include_disabled_users = False
    datasource.store()
    users[1].is_active = False
    users[1].store()

    # by uuid
    assert datasource.get_structured_value('abc0') is None
    assert datasource.get_display_value('abc0') is None
    assert datasource.get_structured_value('abc1') is None
    assert datasource.get_display_value('abc1') is None

    # by id
    assert datasource.get_structured_value('1') is None
    assert datasource.get_display_value('1') is None
    assert datasource.get_structured_value('2') is None
    assert datasource.get_display_value('2') is None


def test_datasource_users_user_formdef(pub):
    pub.user_class.wipe()

    formdef = UserFieldsFormDef(pub)
    formdef.fields = [
        StringField(id='3', label='test', varname='plop'),
    ]
    formdef.store()

    user = pub.user_class(name='John Doe')
    user.form_data = {'3': 'Bar'}
    user.store()

    NamedDataSource.wipe()
    datasource = NamedDataSource(name='foo')
    datasource.data_source = {'type': 'wcs:users'}
    datasource.store()

    assert data_sources.get_items({'type': datasource.slug}) == [
        (
            '3',
            'John Doe',
            '3',
            {
                'user_display_name': 'John Doe',
                'user_email': None,
                'user_f3': 'Bar',
                'user_field_test': 'Bar',
                'user_var_plop': 'Bar',
                'user_admin_access': False,
                'user_backoffice_access': False,
                'id': 3,
                'text': 'John Doe',
            },
        )
    ]
